
# q-expansion of newform 155.2.b.b, downloaded from the LMFDB on 26 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 155
weight = 2
poly_data = [4, 0, 92, 0, 183, 0, 88, 0, 16, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 5, 0, 1, 0, 0, 0, 0, 0, 0], 1], [[0, 22, 5, 34, 1, 11, 0, 1, 0, 0], 2], [[0, -22, 5, -34, 1, -11, 0, -1, 0, 0], 2], [[6, 22, 7, 34, 1, 11, 0, 1, 0, 0], 2], [[-4, -12, -20, -27, -9, -10, -1, -1, 0, 0], 2], [[4, -12, 20, -27, 9, -10, 1, -1, 0, 0], 2], [[0, 54, 0, 127, 0, 68, 0, 14, 0, 1], 4], [[2, -12, 27, -27, 35, -10, 11, -1, 1, 0], 2]]

hecke_ring_character_values = [[32, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [96, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 1, 0], [0, 0, 0, 1, 0, 0, 0, 0, 0, 0], [0, -2, -1, 1, -1, 0, 1, 1, -2, 0], [1, 0, 0, 0, -1, -1, 0, -1, 0, 1], [0, 0, -1, 0, 0, 0, -1, -1, 0, 0], [0, -2, 1, 0, 0, 0, 1, 1, -1, 0], [1, 0, 0, 1, 1, 0, -1, 2, 0, -1], [0, 2, 1, -2, 2, 0, -1, -1, 2, 0], [-3, 0, 0, 0, -1, -1, 0, 1, 0, -1], [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 2, -3, 0, 0, 0, -1, -1, 1, 0], [4, 0, 0, -1, 0, 1, -1, 1, 0, 0], [0, -4, 1, 0, 0, 0, 3, 3, -1, 0], [0, -2, -2, 0, 0, 0, 0, 0, -2, 0], [0, 2, 2, 0, 0, 0, -2, -2, 1, 0], [-2, 0, 0, -1, 0, 1, -3, -1, 0, 4], [0, 0, 0, 0, -2, -2, 2, 0, 0, -2], [0, 2, 3, -2, 2, 0, -1, -1, 2, 0], [1, 0, 0, 3, 1, -2, 1, 0, 0, -1], [0, 2, 0, 2, -2, 0, 0, 0, -1, 0], [-3, 0, 0, -2, -1, 1, 0, -1, 0, 1], [0, 4, 1, -2, 2, 0, 1, 1, 1, 0], [-5, 0, 0, 0, 1, 1, 0, -1, 0, 1], [0, 0, 4, 1, -1, 0, 0, 0, 2, 0], [-1, 0, 0, 1, -3, -4, 1, 0, 0, -1], [0, -2, -4, 1, -1, 0, 0, 0, -2, 0], [0, 2, 2, -3, 3, 0, -2, -2, 2, 0], [-1, 0, 0, 3, 1, -2, 3, 2, 0, -5], [0, -4, 1, 3, -3, 0, 3, 3, -4, 0], [0, 2, -5, 2, -2, 0, -1, -1, 0, 0], [-3, 0, 0, 0, 2, 2, 1, -2, 0, 1], [0, -2, -4, 2, -2, 0, 2, 2, -1, 0], [3, 0, 0, -4, -1, 3, -2, -1, 0, 3], [-6, 0, 0, -2, 3, 5, -1, -3, 0, 4], [-5, 0, 0, 2, -1, -3, -2, 3, 0, -1], [0, 0, 3, -1, 1, 0, 1, 1, 0, 0], [0, 10, -2, -1, 1, 0, -2, -2, 2, 0], [0, 0, 5, 0, 0, 0, 1, 1, -3, 0], [0, -4, 1, -2, 2, 0, -1, -1, 0, 0], [-5, 0, 0, 4, 3, -1, 0, 1, 0, -1], [-4, 0, 0, -4, 0, 4, 0, -2, 0, 2], [6, 0, 0, -1, 1, 2, 0, 4, 0, -4], [0, -4, 6, 1, -1, 0, 2, 2, 0, 0], [0, 12, 5, -6, 6, 0, -5, -5, 12, 0], [3, 0, 0, 0, 3, 3, 2, -1, 0, -1], [-4, 0, 0, 3, -5, -8, 0, 2, 0, -2], [0, -4, 0, 2, -2, 0, 2, 2, -1, 0], [0, 6, 1, -2, 2, 0, -3, -3, 4, 0], [1, 0, 0, -6, 1, 7, -2, -1, 0, 3], [0, 0, 1, -3, 3, 0, -1, -1, -2, 0], [-6, 0, 0, -4, -4, 0, 4, -2, 0, -2], [13, 0, 0, -2, 1, 3, 2, 3, 0, -5], [-4, 0, 0, -4, 2, 6, -4, 0, 0, 4], [0, -4, -5, 1, -1, 0, -3, -3, 4, 0], [0, 0, 4, 0, 0, 0, 4, 4, -5, 0], [-5, 0, 0, 8, 3, -5, 0, -1, 0, 1], [-1, 0, 0, -2, 7, 9, -2, -1, 0, 3], [0, -2, 2, 0, 0, 0, 0, 0, 5, 0], [7, 0, 0, 1, 5, 4, -3, 0, 0, 3], [0, 2, 8, 0, 0, 0, -2, -2, 2, 0], [0, -4, -3, 2, -2, 0, -1, -1, 0, 0], [0, 10, 3, -5, 5, 0, -3, -3, 10, 0], [-6, 0, 0, -1, -6, -5, -1, -3, 0, 4], [0, 2, -2, 2, -2, 0, 0, 0, -3, 0], [0, 4, 0, -1, 1, 0, 0, 0, -2, 0], [2, 0, 0, -6, -6, 0, -2, 0, 0, 2], [0, -6, -7, 2, -2, 0, 5, 5, -7, 0], [0, -2, 1, 2, -2, 0, -1, -1, -2, 0], [16, 0, 0, 2, 1, -1, -1, -1, 0, 2], [0, 8, -3, -2, 2, 0, -9, -9, 8, 0], [-4, 0, 0, -1, 1, 2, -6, -4, 0, 10], [0, 4, -5, 2, -2, 0, -5, -5, 3, 0], [0, 0, 7, -1, 1, 0, 5, 5, 4, 0], [2, 0, 0, 2, -5, -7, 1, 1, 0, -2], [0, 0, 8, 0, 0, 0, 2, 2, -7, 0], [-6, 0, 0, 6, 4, -2, 2, -2, 0, 0], [0, -8, -4, 1, -1, 0, 4, 4, -8, 0], [17, 0, 0, -4, 1, 5, 2, -1, 0, -1], [-11, 0, 0, -2, 1, 3, 0, -7, 0, 7], [8, 0, 0, 3, -2, -5, -3, 5, 0, -2], [4, 0, 0, 5, 2, -3, -1, 1, 0, 0], [-1, 0, 0, -4, 2, 6, -1, 2, 0, -1], [0, -4, -3, 2, -2, 0, -3, -3, 4, 0], [-2, 0, 0, -1, -4, -3, 1, -1, 0, 0], [0, 10, 8, -3, 3, 0, 0, 0, 6, 0], [-10, 0, 0, -4, -4, 0, 4, 0, 0, -4], [0, 0, 1, -2, 2, 0, -1, -1, -4, 0], [-1, 0, 0, 2, 1, -1, 4, -5, 0, 1], [0, 6, -3, 2, -2, 0, 5, 5, -10, 0], [0, -2, -10, 5, -5, 0, 6, 6, -12, 0], [2, 0, 0, -1, -3, -2, 4, -4, 0, 0], [0, 16, 0, -4, 4, 0, -8, -8, 11, 0], [12, 0, 0, 4, 4, 0, 8, 0, 0, -8], [3, 0, 0, -4, -5, -1, 2, 3, 0, -5], [0, 6, -9, 3, -3, 0, -3, -3, 2, 0], [2, 0, 0, 2, 8, 6, -4, 8, 0, -4], [7, 0, 0, -4, 2, 6, 1, 2, 0, -3], [0, 4, -7, -2, 2, 0, -1, -1, 3, 0], [-6, 0, 0, -5, -7, -2, 0, 2, 0, -2], [0, -14, 5, 1, -1, 0, 11, 11, -6, 0], [0, 0, -7, 2, -2, 0, -5, -5, -6, 0], [0, -10, -3, 9, -9, 0, -1, -1, -4, 0], [-16, 0, 0, -2, 8, 10, -4, 2, 0, 2], [-1, 0, 0, 4, -1, -5, 0, 5, 0, -5], [0, -20, -1, 10, -10, 0, 5, 5, -10, 0], [0, 10, -7, -4, 4, 0, -3, -3, 2, 0], [0, 4, -14, 1, -1, 0, -2, -2, -4, 0], [12, 0, 0, 1, -7, -8, 0, 2, 0, -2], [-9, 0, 0, 4, 9, 5, 0, 11, 0, -11], [0, -14, 5, 2, -2, 0, 5, 5, 0, 0], [0, 6, -3, -4, 4, 0, -1, -1, 7, 0], [0, 4, 5, 2, -2, 0, -5, -5, 2, 0], [-10, 0, 0, -8, -16, -8, 4, -10, 0, 6], [-26, 0, 0, 6, 4, -2, 0, 6, 0, -6], [28, 0, 0, 2, 2, 0, -2, 4, 0, -2], [0, 4, 5, -2, 2, 0, -3, -3, 9, 0], [0, -16, -7, 4, -4, 0, 9, 9, -7, 0], [0, 4, 6, -4, 4, 0, 2, 2, 12, 0], [5, 0, 0, 3, 1, -2, 9, 0, 0, -9], [-11, 0, 0, -1, 5, 6, -7, -2, 0, 9], [0, -6, -3, -2, 2, 0, 1, 1, -7, 0], [0, 6, -3, -4, 4, 0, -1, -1, 7, 0], [0, -10, -5, 3, -3, 0, 5, 5, 0, 0], [-10, 0, 0, -1, 2, 3, -5, -7, 0, 12], [-2, 0, 0, 5, -5, -10, -4, -2, 0, 6], [10, 0, 0, 2, 0, -2, -2, 2, 0, 0], [0, 0, 0, 4, -6, -10, -4, 0, 0, 4], [0, -10, 2, -1, 1, 0, 2, 2, -2, 0], [0, 8, 0, 1, -1, 0, -4, -4, 4, 0], [4, 0, 0, 8, 2, -6, 0, 4, 0, -4], [0, -12, -7, 4, -4, 0, 7, 7, -11, 0], [-16, 0, 0, -1, -3, -2, 10, 0, 0, -10], [0, 10, 4, -12, 12, 0, -2, -2, 15, 0], [14, 0, 0, -8, 4, 12, 4, 0, 0, -4], [0, 0, 0, 1, 4, 3, 1, -1, 0, 0], [0, -20, -11, 10, -10, 0, 9, 9, -22, 0], [0, 6, -7, -8, 8, 0, -5, -5, -2, 0], [0, -16, -3, 10, -10, 0, 15, 15, -14, 0], [-21, 0, 0, 4, -5, -9, 0, 3, 0, -3], [2, 0, 0, -2, 5, 7, 3, -1, 0, -2], [15, 0, 0, 8, 5, -3, 0, -1, 0, 1], [0, 8, -12, 0, 0, 0, 0, 0, -5, 0], [0, -4, 6, 0, 0, 0, 6, 6, 3, 0], [2, 0, 0, 8, 4, -4, 8, 4, 0, -12], [17, 0, 0, -4, 8, 12, 7, 8, 0, -15], [0, 4, 8, -8, 8, 0, 0, 0, 0, 0], [0, 8, -1, -6, 6, 0, -11, -11, 4, 0], [5, 0, 0, 2, 13, 11, -4, 3, 0, 1], [0, -8, -5, -4, 4, 0, -3, -3, -1, 0], [0, -8, -1, -5, 5, 0, -3, -3, -4, 0], [3, 0, 0, 4, -7, -11, 10, 1, 0, -11], [0, 4, -8, 2, -2, 0, -2, -2, -7, 0], [0, 2, 9, -1, 1, 0, 3, 3, 12, 0], [0, 10, -2, -3, 3, 0, -10, -10, 18, 0], [-7, 0, 0, 10, -5, -15, -4, -1, 0, 5], [-17, 0, 0, 0, 8, 8, -7, 8, 0, -1], [-8, 0, 0, 6, 6, 0, -2, 8, 0, -6], [0, -8, 1, 6, -6, 0, 11, 11, -4, 0], [20, 0, 0, 12, 4, -8, -6, 8, 0, -2], [0, 4, 0, -8, 8, 0, -10, -10, 13, 0], [0, -26, 0, 6, -6, 0, 10, 10, -9, 0], [0, -14, -15, 4, -4, 0, 1, 1, -8, 0], [-8, 0, 0, 2, -1, -3, 7, -11, 0, 4], [0, 24, -3, -7, 7, 0, -9, -9, 18, 0], [0, 14, -1, -2, 2, 0, -9, -9, -6, 0], [13, 0, 0, 0, 3, 3, 6, -3, 0, -3], [0, -8, -10, 5, -5, 0, -2, -2, -8, 0]]
